<?php

# Personal Weblog 0.9.3
#
# Copyright 2001  Mark Pulford <mark@kyne.com.au>
# This file is subject to the terms and conditions of the GNU General Public
# License. Read the file COPYING found in this archive for details, or
# visit http://www.gnu.org/copyleft/gpl.html
#
# See http://www.kyne.com.au/~mark/software/weblog.php for the lastest
# version.

class Weblog {
	var $admin;
	var $dbh, $dbvar, $dbname;
	var $broken;
	var $valid_cgi;

	# Acceptable cgi variables (when prefix with "wl_")
	# Note these vars are *always* slash escaped, either through
	# "magic quotes" or being explicitly set.
	var $mode, $eid, $tid, $password, $name, $icon, $value, $title, $body;
	var $more, $update, $offset, $search, $topic, $delopt, $before, $start;

	# Print weblog HTML
	function insert()
	{
		if($this->broken) {
			$this->print_error("Cannot insert, not initialised");
			return;
		}
	
		echo $this->getvar("header")."\n";

		$mode_fn = array(
			"more" => "entry_display",
			"edit setup" => "setup_edit",
			"save setup" => "setup_save",
			"edit entry" => "entry_edit",
			"preview entry" => "entry_preview",
			"save entry" => "entry_save",
			"delete entry" => "entry_delete",
			"delete topic" => "topic_delete",
			"change topic" => "topic_update",
			"add topic" => "topic_add",
			"edit topics" => "topic_edit",
			"login" => "login",
			"logout" => "logout"
		);

		# The configurable buttons have varying mode names
		$mode_fn[$this->getvar("search_text")] = "entry_search";
		$rt = $this->getvar("reset_text");
		if($rt)
			$mode_fn[$rt] = "entry_reset";

		if(!isset($this->mode))
			$this->entry_search();
		else if(isset($mode_fn[$this->mode]))
			$this->$mode_fn[$this->mode]();
		else
			$this->print_error("Unknown mode: $mode");

		echo $this->getvar("footer")."\n";
	}

	# Called to ensure the user is logged in, and also displays
	# a login screen where necessary.
	function admin($login = false)
	{
		global $PHP_SELF;

		if(!$this->admin) {
			if($login)
				echo "<h2>Login</h2>\n";
			
			if(isset($this->password))
				$this->print_error("incorrect password");
			else if(!$login)
				$this->print_error("not logged in");

			echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
			$this->print_form_vars(array_diff($this->valid_cgi,array("password")));
			echo "<input type=\"password\" name=\"wl_password\">\n";
			echo "<input type=\"submit\" value=\"Login\">\n";
			echo "</form>\n";
			$this->print_back();
		}

		return $this->admin;
	}

	# Returns a query string containing the current search
	# parameters.
	function search_string()
	{
		$vars = array();
		foreach (array("offset","topic","before", "start") as $v)
			if($this->$v)
				array_push($vars, "wl_$v=".$this->$v);
		if($this->search) {
			$s = str_replace(" ", "+", $this->search);
			array_push($vars, "wl_search=$s");
		}
		return join('&', $vars);
	}

	# Print a link back to the main page while maintaining
	# current search parameters.
	function print_back()
	{
		global $PHP_SELF;

		$q = $this->search_string();
		if($q)
			$q = "?$q";
		$bl = $this->getvar("backlink");
		echo str_replace("@URL@", "$PHP_SELF$q", $bl);
	}

	# Reset the weblog last modified time.
	# Called whenever an entry/topic is added/changed/deleted.
	function update_last_modified()
	{
		if($this->getvar("last_modified") != -1)
			$this->setvar("last_modified", time());
	}

	# CGI mode: "change topic"
	# Uses: tid, name, icon
	function topic_update()
	{
		echo "<h2>Updating topic</h2>\n";

		if(!$this->admin())
			return;

		if(!$this->tid) {
			$this->print_error("no topic selected");
			$this->print_back();
			return;
		}

		if($this->name)
			$terms = "name='{$this->name}'";
		if($this->icon) {
			if($terms)
				$terms .= ", ";
			$terms .= "icon='{$this->icon}'";
		}
		if(!$terms) {
			$this->print_error("At least one of icon or name must be set");
			$this->print_back();
			return;
		}

		$this->update_last_modified();

		if(!$this->dbquery("UPDATE topics SET $terms WHERE tid={$this->tid}"))
			return;
		if(mysql_affected_rows())
			echo "<p><b>Topic changed</b>\n";
		else
			echo "<p><b>Topic not changed</b>\n";

		$this->print_back();
	}

	# CGI mode: "add topic"
	# Uses: name, icon
	function topic_add()
	{
		echo "<h2>Adding topic</h2>\n";

		if(!$this->admin())
			return;

		if(!$this->name or !$this->icon) {
			$this->print_error("Both name and icon must be set");
			$this->print_back();
			return;
		}

		$this->update_last_modified();

		if(!$this->dbquery("INSERT INTO topics VALUES (NULL, '".addslashes($this->name)."', '".addslashes($this->icon)."')"))
			return;
		if(mysql_affected_rows())
			echo "<p><b>Topic added</b>\n";
		else
			$this->print_error("failed to add topic");

		$this->print_back();
	}

	# CGI mode: "delete topic"
	# Uses: tid, delopt
	function topic_delete()
	{
		echo "<h2>Deleting topic</h2>\n";

		if(!$this->admin())
			return;

		if(!$this->tid) {
			$this->print_error("no topic selected");
			$this->print_back();
			return;
		}

		if($this->tid == $this->delopt) {
			$this->print_error("Cannot move entries into the topic about to be deleted!");
			$this->print_back();
			return;
		}

		# Verify "delopt" exists unless it is zero
		# 0 => use "tid" for a dummy lookup.
		$v = $this->delopt;
		if(!$v)
			$v = $this->tid;
		$tname = $this->topic_verify($v);
		if(!$tname)
			return;


		$this->update_last_modified();

		if($this->delopt) {
			$q = "UPDATE entries SET tid={$this->delopt} WHERE tid={$this->tid}";
			echo "<p>Moving entries to $tname\n";
		} else {
			$q = "DELETE FROM entries WHERE tid={$this->tid}";
			echo "<p>Deleting any existing entries for $tname\n";
		}

		if(!$this->dbquery($q))
			$this->dbunlock();
		else if(!$this->dbquery_unlock("DELETE FROM topics WHERE tid={$this->tid}", $aff))
			return;

		if($aff)
			echo "<p><b>Topic deleted</b>\n";
		else
			$this->print_error("topic not found to delete");

		$this->print_back();
	}

	# Print hidden fields to maintain current cgi variable
	# values through html form
	function print_form_vars($cgi_names)
	{
		foreach($cgi_names as $v) {
			if(isset($this->$v)) {
				if(is_array($this->$v)) {
					foreach ($this->$v as $k2 => $v2)
						echo "<input type=\"hidden\" name=\"wl_{$v}[{$k2}]\" value=\"".htmlspecialchars($v2)."\">\n";
				} else {
					echo "<input type=\"hidden\" name=\"wl_$v\" value=\"".htmlspecialchars($this->$v)."\">\n";
				}
			}
		}
	}

	# Maintain search parameters through html form
	function print_search_details()
	{
		$this->print_form_vars(array("offset", "topic", "search", "before", "start"));
	}

	# CGI mode: "edit topics"
	# Uses: none
	function topic_edit()
	{
		global $PHP_SELF;

		echo "<h2>Edit topics</h2>\n";

		if(!$this->admin())
			return;

		$r = $this->dbquery("SELECT * FROM topics");
		if(!$r)
			return;
		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		echo "<table border=1 cellpadding=3>\n";
		echo "<tr><th>name</th><th>icon</th><th>sel</th></tr>\n";
		while($a = $this->dbfetch($r)) {
			echo "<tr>";
			echo "<td>".$a["name"]."</td>\n";
			echo "<td><img src=\"{$a["icon"]}\"><a href=\"".$a["icon"]."\">".$a["icon"]."</a></td>\n";
			echo "<td><input type=\"radio\" name=\"wl_tid\" value=\"{$a["tid"]}\"></td>";
			echo "</tr>\n";
		}
		echo "</table>\n";
		echo "<input type=\"radio\" name=\"wl_tid\" value=\"\" checked> select none<br>\n";

		echo "Name: <input type=\"text\" name=\"wl_name\" maxlength=\"80\"> (add/change option)<br>\n";
		echo "Icon: <input type=\"text\" name=\"wl_icon\" maxlength=\"128\"> (add/change option)<br>\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"change topic\">\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"add topic\"><br>\n";
		echo "Move or delete entries currently in topic: ";
		echo $this->topic_selectbox("delopt", -1, "Delete");
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"delete topic\">\n";
		echo "</form>\n";
		$this->print_back();
	}

	# CGI mode: "save setup"
	# Uses: value
	function setup_save()
	{
		echo "<h2>Updating configuration</h2>\n";

		if(!$this->admin())
			return;

		foreach($this->value as $k => $v)
			$this->setvar($k, $v);

		echo "<p><b>Configuration saved</b>\n";

		$this->print_back();
	}

	# Display an image form passing several hidden fields
	# image_link("next", "gfx/next.gif", array("mode"=>"next", "a"=>"b"));
	# $name must not be a potential CGI variable
	function image_link($name, $gfx, $vars)
	{
		global $PHP_SELF;

		$il = "<form action=\"$PHP_SELF\" method=\"POST\">";
		foreach($vars as $k => $v)
			$il .= "<input type=\"hidden\" name=\"wl_$k\" value=\"$v\">";
		$il .= "<input type=\"image\" src=\"$gfx\" name=\"$name\" alt=\"$name\" ".$this->getvar("nav_icon_attrs").">";
		$il .= "</form>";

		return $il;
	}

	# CGI mode: "edit setup"
	# Uses: none
	function setup_edit()
	{
		global $PHP_SELF;

		echo "<h2>Edit weblog setup</h2>\n";

		if(!$this->admin())
			return;

		$r = $this->dbquery("select * from vars where editable!='no' order by name asc");
		if(!$r)
			return;

		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		while($a = $this->dbfetch($r)) {
			echo "<h2>{$a["name"]}</h2>\n";
			if($a["help"])
				echo "<p>{$a["help"]}\n";
			echo "<p>";
			if($a["editable"] == "single") {
				echo "<input type=\"text\" name=\"wl_value[{$a["name"]}]\" value=\"".htmlspecialchars($a["value"])."\" size=\"60\">\n";
			} else {
				echo "<textarea name=\"wl_value[{$a["name"]}]\" rows=\"5\" cols=\"60\">".htmlspecialchars($a["value"])."</textarea>\n";
			}
		}
		echo "<p><input type=\"submit\" name=\"wl_mode\" value=\"save setup\">\n";
		echo "</form>\n";
		$this->print_back();
	}

	# Load a weblog entry overriding current title, tid, body, more
	# values
	function entry_load($eid)
	{
		$r = $this->dbquery("SELECT * FROM entries WHERE eid=$eid");
		if(!$r)
			return false;
		if(!mysql_num_rows($r)) {
			$this->print_error("entry not found");
			$this->print_back();
			return false;
		}

		$a = $this->dbfetch($r);
		foreach (array("title","tid","body","more") as $v)
			$this->$v = $a[$v];

		return true;
	}

	# CGI mode: "edit entry"
	# Uses: eid, tid, title, body, more, update
	# Edit a DB entry. The entry can be given from CGI variables,
	# start blank, or be loaded from the DB.
	function entry_edit()
	{
		global $PHP_SELF;

		echo "<h2>Edit entry</h2>\n";

		if(!$this->admin())
			return;

		if(isset($this->eid) && !isset($this->tid))
			if(!$this->entry_load($this->eid))
				return;

		echo "<form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		if(isset($this->eid))
			echo "<input type=\"hidden\" name=\"wl_eid\" value=\"{$this->eid}\">\n";
		echo "Title: <input type=\"text\" name=\"wl_title\" value=\"".htmlspecialchars($this->title)."\" maxlength=\"80\" size=\"60\"><br>\n";
		echo "Topic:&nbsp;";
		echo $this->topic_selectbox("tid", $this->tid, "");
		echo "<br>\n";
		echo "Body:<br>\n";
		echo "<textarea name=\"wl_body\" rows=\"10\" cols=\"60\">".htmlspecialchars($this->body)."</textarea><br>\n";
		echo "More:<br>\n";
		echo "<textarea name=\"wl_more\" rows=\"10\" cols=\"60\">".htmlspecialchars($this->more)."</textarea><br>\n";
		# Only offer "update" when editing an existing entry
		if(isset($this->eid)) {
			echo "<input type=\"checkbox\" name=\"wl_update\" value=\"yes\"";
			if($this->update == "yes")
				echo " checked";
			echo "> Update timestamp<br>\n";
		}
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"preview entry\">\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"save entry\"><br>\n";
		echo "</form>\n";
		$this->print_back();
	}

	# CGI mode: "preview entry"
	# Uses: eid, tid, title, body, more, update
	function entry_preview()
	{
		global $PHP_SELF;

		echo "<h2>Preview entry</h2>\n";

		if(!$this->admin())
			return;

		if($this->eid)
			$r = $this->dbquery("SELECT created,updated,name,icon FROM entries,topics WHERE eid={$this->eid} AND topics.tid={$this->tid}");
		else
			$r = $this->dbquery("SELECT name,icon FROM topics WHERE tid={$this->tid}");
		if(!$r)
			return;
		$a = $this->dbfetch($r);

		# New entry? Set times. Otherwise check for update.
		$tm = time();
		if(!$a["created"]) {
			$a["created"] = $tm;
			$a["updated"] = $tm;
		} else if($this->update == "yes") {
			$a["updated"] = $tm;
		}

		# Set remaining vars
		foreach (array("eid","tid","title","body","more") as $v)
			$a[$v] = $this->$v;

		echo $this->subst_tokens($this->getvar("more"), $a)."\n";

		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		foreach (array("eid","tid","title","body","more","update") as $v)
			if(isset($this->$v))
				echo "<input type=\"hidden\" name=\"wl_$v\" value=\"".htmlspecialchars($this->$v)."\">\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"edit entry\">\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"save entry\">\n";
		echo "</form>\n";

		$this->print_back();
	}

	# CGI mode: "save entry"
	# Uses: eid, tid, title, body, more, update
	function entry_save()
	{
		echo "<h2>Saving entry</h2>\n";

		if(!$this->admin())
			return;

		if(!$this->topic_verify($this->tid))
			return;
		
		$this->update_last_modified();

		$tm = time();
		if($this->eid) {
			$q = "UPDATE entries SET title='".addslashes($this->title)."', tid={$this->tid}, body='".addslashes($this->body)."', more='".addslashes($this->more)."'";
			if($this->update == "yes")
				$q .= ", updated=$tm";
			$q .= " WHERE eid={$this->eid}";
		} else {
			$q = "INSERT INTO entries VALUES (NULL, '".addslashes($this->title)."', {$this->tid}, $tm, $tm, '".addslashes($this->body)."','".addslashes($this->more)."')";
		}

		if(!$this->dbquery_unlock($q, $aff))
			return;

		if($this->eid && !$aff) {
			echo "<p><b>Entry not changed</b>\n";
		} else {
			echo "<p><b>Entry saved</b>\n";

			# Call add/edit entry hook
			if(!$this->entry_save_local(!isset($this->eid), $this->title, $this->body, $this->more, $tm))
				$this->print_error("local hook failed");
		}

		$this->print_back();
	}

	# Hook to extend functionality when adding/updating an entry
	# $new is true when the entry has been added.
	function entry_save_local($new, $title, $body, $more, $tm)
	{
		return true;
	}

	# CGI mode: "delete entry"
	# Uses: eid
	function entry_delete()
	{
		echo "<h2>Deleting entry</h2>\n";

		if(!$this->admin())
			return;

		$this->update_last_modified();

		if(!$this->dbquery("DELETE FROM entries WHERE eid={$this->eid}"))
			return;
		if(mysql_affected_rows() == 1)
			echo "<p><b>Entry deleted</b>\n";
		else
			$this->print_error("entry not found to delete");

		$this->print_back();
	}

	# CGI mode: special ("reset_text" variable)
	# Uses: none
	function entry_reset()
	{
		unset($this->topic);
		unset($this->search);
		unset($this->offset);
		unset($this->before);
		unset($this->start);

		$this->entry_search();
	}

	# CGI mode: special ("search_text" variable, default)
	# Uses: topic, offset, search, before, start
	function entry_search()
	{
		$sep = $this->getvar("separator");
		$max = $this->getvar("max_articles");

		$q = "SELECT * from entries,topics WHERE entries.tid=topics.tid ";

		if($this->start) {
			$r = $this->dbquery("SELECT updated from entries WHERE eid={$this->start}");
			if(!$r)
				return;
			if(!mysql_num_rows($r)) {
				echo "<b>No entries found.</b>\n";
				echo "$sep\n";
				$this->print_control($this->topic, $this->search, $this->offset, false);
				return;
			}
			$arr = $this->dbfetch($r);
			$q .= "AND updated<=".$arr["updated"]." ";
		}
		if($this->before)
			$q .= "AND updated<={$this->before} ";
		if(0<$this->topic)
			$q .= "AND topics.tid={$this->topic} ";
		if($this->search) {
			$keywords = split(" +", $this->search);
			foreach($keywords as $k) {
				$sk = addslashes($k);	# currently redundant
				$q .= "AND (entries.title LIKE '%$sk%' OR entries.body LIKE '%$sk%' OR entries.more LIKE '%$sk%') ";
			}
		}
		$q .= "ORDER BY updated DESC ";
		$off = $this->offset;
		if(0<$max) {
			if(!$off)
				$off = 0;

			# Search for an extra entry. Do not display the extra
			# entry, it is used to determine whether a next icon
			# is needed.
			$q .= "LIMIT $off, ".($max+1)." ";
		}
		$r = $this->dbquery($q);
		if(!$r)
			return;
		$rows = mysql_num_rows($r);

		if($rows > $max) {
			$show_next = 1;
			$rows = $max;
		}

		if($rows == 0) {
			echo "<b>No entries found.</b>\n";
			echo "$sep\n";
		} else {
			$this->print_entries($r, $rows);
		}

		$this->print_control($this->topic, $this->search, $this->offset, $show_next);
	}

	# CGI mode: "more"
	# Uses: eid
	function entry_display()
	{
		$r = $this->dbquery("SELECT * FROM entries,topics WHERE eid={$this->eid} and entries.tid=topics.tid");
		if(!$r)
			return;
		if(!mysql_num_rows($r)) {
			echo "<b>No entry found.</b>\n";
		} else {
			$mf = $this->getvar("more");
			echo $this->subst_tokens($mf, $this->dbfetch($r))."\n";
			$this->print_back();
		}
		echo $this->getvar("separator")."\n";
		$this->print_control(0, "", 0, 0);
	}

	# Print per entry admin menu maintaining search parameters
	function entry_print_admin($eid)
	{
		global $PHP_SELF;

		$q = $this->search_string();
		if($q)
			$q = "&$q";
		$adm = "<p><b>Admin</b>: ";
		$adm .= "<a href=\"$PHP_SELF?wl_mode=edit+entry&wl_eid=$eid$q\">[edit]</a>\n";
		$adm .= "<a href=\"$PHP_SELF?wl_mode=delete+entry&wl_eid=$eid$q\">[delete]</a>\n";

		return $adm;
	}

	# Ensure all non string CGI variables are clean.
	# Strings are always slash escaped before passing to MySQL
	function sanitize_input()
	{
		# name, icon, title, body, more, value
		# - Used in db searches, but they are slash escaped
		# mode, password, update
		# - not used in db searches

		# Sanitize. remove non alphanumeric. shrink spaces.
		# remove prefix/suffix spaces.
		if(isset($this->search)) {
			$this->search = eregi_replace("[^a-z0-9 ]", "", $this->search);
			$this->search = eregi_replace(" +", " ", $this->search);
			$this->search = trim($this->search);
			# Only keep first 3 keywords
			# Do this here so the search box contains the updated
			# search string.
			if(ereg("([^ ]+ [^ ]+ [^ ]+) .*", $this->search, $reg)) {
				$this->search = $reg[1];
			}
		}
		# Ensure numeric CGI vars are valid.
		foreach (array("topic","delopt","tid","eid","offset","before","start") as $v)
			if(isset($this->$v)) {
				$this->$v = eregi_replace("[^0-9]", "", $this->$v);
				# Ensure a blanked var will return 0 as a string
				if(!$this->$v)
					$this->$v = 0;
			}
	}

	# Print $rows entries from the MySQL search result $r
	function print_entries($r, $rows)
	{
		$af = $this->getvar("article");
		$sep = $this->getvar("separator");

		for($i=0; $i<$rows; $i++) {
			$arr = $this->dbfetch($r);
			echo $this->subst_tokens($af, $arr)."\n";
			echo "$sep\n";
		}
	}

	# Substitute an entry into a template via tokens
	function subst_tokens($format, $entry)
	{
		global $PHP_SELF;

		# Create morelink for @MORE@
		if($entry["more"]) {
			$ml = $this->getvar("morelink");
			$q = $this->search_string();
			if($q)
				$q = "&$q";
			$ml = str_replace("@URL@", "$PHP_SELF?wl_mode=more&wl_eid={$entry["eid"]}$q", $ml);
		} else {
			$ml = "";
		}

		if($entry["created"] != $entry["updated"]) {
			$upd = $this->getvar("update_text");
			$upd = str_replace("@DATE@", date($this->getvar("updated_date"), $entry["updated"]), $upd);
		} else {
			$upd = "";
		}

		if($this->admin)
			$al = $this->entry_print_admin($entry["eid"]);
		else
			$al = "";

		$q = $this->search_string();
		if($q)
			$q = "?$q";
		$bl = $this->getvar("backlink");
		$bl = str_replace("@URL@", "$PHP_SELF$q", $bl);

		$trans = array(
			"@TITLE@" => $entry["title"],
			"@CREATED@" => date($this->getvar("created_date"), $entry["created"]),
			"@UPDATED@" => $upd,
			"@ICON@" => "<a href=\"$PHP_SELF?wl_topic={$entry["tid"]}\"><img src=\"{$entry["icon"]}\" alt=\"Topic: {$entry["name"]}\" align=\"left\" ".$this->getvar("topic_icon_attrs")."></a>",
			"@BODY@" => $entry["body"],
			"@ID@" => $entry["eid"],
			"@MORELINK@" => $ml,
			"@BACKLINK@" => $bl,
			"@MORE@" => $entry["more"],
			"@ADMIN@" => $al);

		return strtr($format, $trans);
	}

	# Print a next/previous nav icon
	function offset_button($name, $var, $off)
	{
		if($off < 0)
			$off = 0;
		return $this->image_link("[$name]", $this->getvar($var), array("search"=>$this->search, "topic"=>$this->topic, "offset"=>$off, "before"=>$this->before, "start"=>$this->start));
	}

	# Print a blank nav icon placeholder
	function blank_icon()
	{
		$bi = "<img src=\"";
		$bi .= $this->getvar("blank_icon");
		$bi .= "\" alt=\"\" ".$this->getvar("nav_icon_attrs").">";

		return $bi;
	}

	# Show search box, topic select, ...
	function print_control($topic, $search, $offset, $show_next)
	{
		global $PHP_SELF;

		$max = $this->getvar("max_articles");

		$bar = "<table><tr><td>";

		if($offset > 0)
			$bar .= $this->offset_button("previous", "prev_icon", $offset - $max);
		else
			$bar .= $this->blank_icon();
		$bar .= "</td><td>";
		$bar .= "<form method=\"POST\" action=\"$PHP_SELF\">\n";
		$bar .= "&nbsp;".$this->getvar("topic_text");
		$ts = $this->topic_selectbox("topic", $topic, "All");
		if(!$ts)
			return;
		$bar .= $ts;
		$bar .= "&nbsp;";
		$bar .= "<input type=\"text\" name=\"wl_search\" value=\"$search\">&nbsp;";
		$bar .= "<input type=\"submit\" name=\"wl_mode\" value=\"".$this->getvar("search_text")."\">&nbsp;";
		$rt = $this->getvar("reset_text");
		if($rt)
			$bar .= "<input type=\"submit\" name=\"wl_mode\" value=\"$rt\">&nbsp;";
		$bar .= "</form>";
		$bar .= "</td><td>";
		if($show_next)
			$bar .= $this->offset_button("next", "next_icon", $offset + $max);
		else
			$bar .= $this->blank_icon();
		$bar .= "</td></tr></table>";

		if($this->admin) {
			$q = $this->search_string();
			if($q)
				$q = "&$q";
			$bar .= "<p><b>Admin</b>:\n";
			$bar .= "<a href=\"$PHP_SELF?wl_mode=edit+entry$q\">[add entry]</a>\n";
			$bar .= "<a href=\"$PHP_SELF?wl_mode=edit+topics$q\">[edit topics]</a>\n";
			$bar .= "<a href=\"$PHP_SELF?wl_mode=edit+setup$q\">[edit setup]</a>\n";
			$bar .= "<a href=\"$PHP_SELF?wl_mode=logout$q\">[log out]</a>\n";
		}

		echo str_replace("@BAR@", $bar, $this->getvar("control"));
	}

	function dbquery($sql)
	{
		# Must ensure correct DB is selected. If some other PHP
		# code has run it might have changed the db in between
		# Weblog() and insert().
		if(!mysql_select_db($this->dbname)) {
			$this->print_error("cannot select correct database");
			return false;
		}

		$result = @mysql_query($sql, $this->dbh);
		if($result) {
			return $result;
		} else {
			$this->print_error("query failed: ".mysql_error());
			$this->print_back();
			return false;
		}
	}

	# Ensure any magic quotes have been stripped.
	# Always use dbfetch() to get values from a MySQL query result
	function dbfetch($r)
	{
		return $this->mq_strip(mysql_fetch_array($r), "runtime");
	}

	# Unlock the database
	function dbunlock()
	{
		if(!$this->dbquery("UNLOCK TABLES")) {
			$this->print_error("CRITICAL! Failed to unlock tables");
			return false;
		}

		return true;
	}

	# Print an error message.
	# Exit if the error occured before the page was partially displayed
	function print_error($err)
	{
		$h = headers_sent();
	
		if($err)
			$err = ": $err";
		if(!$h) {
			echo "<html><head><title>Critical error</title></head>\n";
			echo "<body>";
		}
		echo "<p><b>ERROR</b>$err\n";
		if(!$h) {
			echo "</body></html>\n";
			exit;
		}
	}

	# Verify the topic exists.
	# NOTE: It leaves the table locked!
	# Locking both entries and topics for write is overkill, but this
	# code does not get used much.
	function topic_verify($tid)
	{
		if(!$tid)
			return "";
	
		if(!$this->dbquery("LOCK TABLES entries WRITE, topics WRITE, vars WRITE"))
			return "";

		$r = $this->dbquery("SELECT tid,name FROM topics WHERE tid=$tid");
		if(!$r || !mysql_num_rows($r)) {
			$this->print_error("Topic not found.");
			$this->print_back();
			$this->dbunlock();
			return "";
		}

		$a = $this->dbfetch($r);
		return $a["name"];
	}

	function dbquery_unlock($sql, &$affected)
	{
		$r = $this->dbquery($sql);

		# Ugly, but if we do not we lose the information.
		$affected = mysql_affected_rows();
		
		if(!$this->dbunlock())
			return false;

		return $r;
	}

	# Variables are assumed to exist and are required to run.
	function getvar($name)
	{
		if(isset($this->dbvar) && isset($this->dbvar[$name]))
			return $this->dbvar[$name];

		$this->print_error("fatal, cannot load variable $name");
		exit;
	}

	# Update a weblog variable
	function setvar($name, $value)
	{
		if($this->getvar($name) == $value)
			return true;

		$this->dbvar[$name] = $value;

		$q = "UPDATE vars SET value='".addslashes($value)."' WHERE name='".addslashes($name)."'";
		if(!$this->dbquery($q))
			return false;
		if(mysql_affected_rows() != 1) {
			$this->print_error("Cannot set $name/$value");
			return false;
		}

		return true;
	}

	# Returns the form HTML necessary to select between topics
	function topic_selectbox($cgi, $t, $field)
	{
	        $r = $this->dbquery("SELECT tid,name from topics ORDER BY name");
		if(!$r)
			return "";

	        $html = "<select name=\"wl_$cgi\">\n";
		if($field)
		        $html .= "<option value=\"0\">$field\n";
	        while($a = $this->dbfetch($r)) {
	                $html .= "<option value=\"{$a["tid"]}\"";
	                if($t == $a["tid"])
	                        $html .= " SELECTED";
	                $html .= ">".$a["name"]."\n";
	        }
	        $html .= "</select>";

		return $html;
	}

	# CGI mode: "login"
	# Uses: none
	#
	# If already logged in show the main page
	function login()
	{
		global $PHP_SELF;

		if(!$this->admin(true))
			return;

		$this->entry_search();
	}

	# CGI mode: "logout"
	# Uses: none
	#
	# Logout and display main page.
	function logout()
	{
		$this->setvar("login_expire", -1);
		$this->admin = false;
		$this->entry_search();
	}

	# Returns true if currently logged in via a cookie
	function current_login()
	{
		global $HTTP_COOKIE_VARS;

		$t = $this->getvar("login_expire");
		if($t < time())
			return false;

		if($HTTP_COOKIE_VARS["sess"] != $this->getvar("login_cookie"))
			return false;

		return true;
	}

	# Cache DB vars to prevent many queries
	function load_db_vars()
	{
		$r = $this->dbquery("SELECT name,value FROM vars");
		if(!$r || !mysql_num_rows($r))
			$this->print_error("cannot load variable");

		while($a = $this->dbfetch($r))
			$this->dbvar[$a["name"]] = $a["value"];
	}

	# Strip magic quotes escapes from a string
	function mq_strip_escape($v)
	{
		if(ini_get("magic_quotes_sybase") == 1)
			return ereg_replace("''", "'", $v);
		else
			return stripslashes($v);
	}

	# Depending on the data source strip magic quotes if they exist
	# Valid types: "gpc", "runtime"
	function mq_strip($cv, $mq = 'gpc')
	{
		if(!isset($cv))
			return $cv;

		$fn = "get_magic_quotes_$mq";
		if(!$fn())
			return $cv;

		if(!is_array($cv))
			return $this->mq_strip_escape($cv);

		foreach($cv as $k => $v)
			$new[$k] = $this->mq_strip_escape($v);

		return $new;
	}

	# Store any GET/POST variables in the class.
	# POST has precedence over GET
	function load_cgi_vars()
	{
		global $HTTP_GET_VARS, $HTTP_POST_VARS;

		$arr = array_merge($HTTP_GET_VARS, $HTTP_POST_VARS);

		foreach($arr as $k => $v)
			if(ereg("^wl_(.+)", $k, $r) && in_array($r[1], $this->valid_cgi))
				$this->$r[1] = $this->mq_strip($v);
	}

	# Start a login session and send/update browser cookie
	function enter_session($sess)
	{
		$exp = time() + $this->getvar("login_timeout");
		setcookie("sess", $sess, $exp);
		$this->setvar("login_cookie", $sess);
		$this->setvar("login_expire", $exp);
		$this->admin = true;
	}

	# The Weblog class must be instantiated at the top of the page
	# before any data is sent. This way it can modify the HTTP headers.
	#
	# Checks for a valid new or current login and updates the session cookie
	# and expiry.
	# Sets the class variable $admin to true on a successful login.
	function Weblog($dbname, $dbhost, $dbuser, $dbpass)
	{
		$this->valid_cgi = array(
			"mode","topic","search","offset","before","start",
			"eid","tid","name","icon","value",
			"title","body","more",
			"password","update","delopt");

		if(headers_sent()) {
			$this->print_error("Weblog constructor must be called before the headers have been sent");
			$this->broken = 1;
			return;
		}

		$this->dbh = @mysql_connect($dbhost,$dbuser,$dbpass);
		if(!$this->dbh)
			$this->print_error("Unable to connect to MySQL server. Check Weblog() arguments.");

		$this->dbname = $dbname;

		$this->load_cgi_vars();
		$this->sanitize_input();
		$this->load_db_vars();

		# Check for login status
		if($this->current_login())
			$this->enter_session($this->getvar("login_cookie"));
		else if($this->password == $this->getvar("password"))
			$this->enter_session($this->gen_session_key());
		else
			$this->admin = false;

		$lm = $this->getvar("last_modified");
		if($lm != -1)
			header("Last-Modified: ".gmdate("D, d M Y H:i:s", $lm)." GMT");
	}

	# Attempt to generate a unique session cookie value
	function gen_session_key()
	{
		mt_srand( ((double)microtime() + posix_getpid()) * 1000000 );
		return mt_rand().mt_rand().mt_rand();
	}
}

?>
